//#############################################################################
//
// FILE:   ipc_ex1_basic_cm.c
//
// TITLE:  IPC example with interrupt
//
//! \addtogroup driver_cm_c28x_dual_example_list
//! <h1> IPC basic message passing example with interrupt </h1>
//!
//! This example demonstrates how to configure IPC and pass information from
//! C28x to CM core without message queues
//! It is recommended to run the C28x1 core first, followed by the CM core.
//!
//! \b External \b Connections \n
//!  - None.
//!
//! \b Watch \b Variables \n
//!  - None.
//!
//
//#############################################################################
// $TI Release: F2838x Support Library v3.02.00.00 $
// $Release Date: Tue May 26 17:21:56 IST 2020 $
// $Copyright:
// Copyright (C) 2020 Texas Instruments Incorporated - http://www.ti.com/
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
// 
//   Redistributions of source code must retain the above copyright 
//   notice, this list of conditions and the following disclaimer.
// 
//   Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the 
//   documentation and/or other materials provided with the   
//   distribution.
// 
//   Neither the name of Texas Instruments Incorporated nor the names of
//   its contributors may be used to endorse or promote products derived
//   from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// $
//###########################################################################

//
// Included Files
//
#include "cm.h"
#include "ipc.h"

//
// Defines
//
#define IPC_CMD_READ_MEM   0x1001
#define IPC_CMD_RESP       0x2001

#define TEST_PASS          0x5555
#define TEST_FAIL          0xAAAA
__interrupt void cmnmiISR(void);


//
// IPC ISR for Flag 0.
// C28x core sends data without message queue using Flag 0
//
void IPC_ISR0()
{
    int i;
    uint32_t command, addr, data;
    bool status = false;

    //
    // Read the command
    //
    IPC_readCommand(IPC_CM_L_CPU1_R, IPC_FLAG0, IPC_ADDR_CORRECTION_ENABLE,
                    &command, &addr, &data);

    if(command == IPC_CMD_READ_MEM)
    {
        status = true;

        //
        // Read and compare data
        //
        for(i=0; i<data; i++)
        {
            if(*((uint32_t *)addr + i) != i)
                status = false;
        }
    }

    //
    // Send response to C28x core
    //
    if(status)
    {
        IPC_sendResponse(IPC_CM_L_CPU1_R, TEST_PASS);
    }
    else
    {
        IPC_sendResponse(IPC_CM_L_CPU1_R, TEST_FAIL);
    }

    //
    // Acknowledge the flag
    //
    IPC_ackFlagRtoL(IPC_CM_L_CPU1_R, IPC_FLAG0);
}

//
// Main
//
short TrigerDelay = 0;
short debCount = 0;
void main(void)
{

    //
    // Initialize device clock and peripherals
    //
    CM_init();

//    //
//    // Clear any IPC flags if set already
//    //
//    IPC_clearFlagLtoR(IPC_CM_L_CPU1_R, IPC_FLAG_ALL);
//
//    //
//    // Enable IPC interrupts
//    //
//    IPC_registerInterrupt(IPC_CM_L_CPU1_R, IPC_INT0, IPC_ISR0);
//
//    //
//    // Synchronize both the cores.
//    //
//    IPC_sync(IPC_CM_L_CPU1_R, IPC_FLAG31);

    // The following configuration leads to 255/(25M/512/64) = 334 ms.
    // Enable the watchdog.
    Interrupt_registerHandler(FAULT_NMI,cmnmiISR);
    Interrupt_enable(FAULT_NMI);
    SysCtl_clearAllNMIFlags();
    SysCtl_setWatchdogPredivider( SYSCTL_WD_PREDIV_512 );
    SysCtl_setWatchdogPrescaler( SYSCTL_WD_PRESCALE_64 );
    SysCtl_enableWatchdog();
    SysCtl_setNMIWatchdogPeriod( 100 );
    SysCtl_enableGlobalNMI();

    //
    // Loop forever. Wait for IPC interrupt
    //
    while( 1 ) {
        // Reset watchdog timer
        if( !TrigerDelay ) {

            SysCtl_serviceWatchdog();
        }

        debCount++;
    }

}

short Wdstatus = 0;
short NMIFlagStatus = 0;
__interrupt void cmnmiISR(void)
{
   // Get the WWD status and clear it
   Wdstatus = SysCtl_getWatchdogStatus();
//  SysCtl_clearWatchdogStatus();

   // NMI was handled after the WWD expired
   NMIFlagStatus = SysCtl_getNMIFlagStatus();
//  SysCtl_clearAllNMIFlags();

}

//
// End of File
//
